@@ -1,5 +1,7 @@
-/*	$OpenBSD: tftp.c,v 1.4 1997/08/06 06:43:45 deraadt Exp $	*/
-/*	$NetBSD: tftp.c,v 1.5 1995/04/29 05:55:25 cgd Exp $	*/
+/* $Id: tftp.c,v 1.1.1.1 1999/09/26 06:32:42 hpa Exp $ */
+
+/* $OpenBSD: tftp.c,v 1.4 1997/08/06 06:43:45 deraadt Exp $	*/
+/* $NetBSD: tftp.c,v 1.5 1995/04/29 05:55:25 cgd Exp $	*/
 
 /*
  * Copyright (c) 1983, 1993
@@ -35,10 +37,9 @@
  */
 
 #ifndef lint
-#if 0
-static char sccsid[] = "@(#)tftp.c	8.1 (Berkeley) 6/6/93";
-#endif
-static char rcsid[] = "$OpenBSD: tftp.c,v 1.4 1997/08/06 06:43:45 deraadt Exp $";
+/* static char sccsid[] = "@(#)tftp.c	8.1 (Berkeley) 6/6/93"; */
+/* static char rcsid[] = "$OpenBSD: tftp.c,v 1.4 1997/08/06 06:43:45 deraadt Exp $"; */
+static const char *rcsid = "tftp-hpa $Id: tftp.c,v 1.1.1.1 1999/09/26 06:32:42 hpa Exp $";
 #endif /* not lint */
 
 /* Many bug fixes are from Jim Guyton <guyton@rand-unix> */
@@ -64,7 +65,13 @@ static char rcsid[] = "$OpenBSD: tftp.c,v 1.4 1997/08/06 06:43:45 deraadt Exp $"
 #include "extern.h"
 #include "tftpsubs.h"
 
-extern	int errno;
+void bsd_signal(int, void (*)(int));
+
+#ifndef HAVE_SIGSETJMP
+#define sigsetjmp(x,y)  setjmp(x)
+#define siglongjmp(x,y) longjmp(x,y)
+#define sigjmp_buf jmp_buf
+#endif
 
 extern  struct sockaddr_in peeraddr;	/* filled in by main */
 extern  int     f;			/* the opened socket */
@@ -83,8 +90,8 @@ extern  int     maxtimeout;
 #define PKTSIZE    SEGSIZE+4
 char    ackbuf[PKTSIZE];
 int	timeout;
-jmp_buf	toplevel;
-jmp_buf	timeoutbuf;
+sigjmp_buf	toplevel;
+sigjmp_buf	timeoutbuf;
 
 static void nak(int);
 static int makerequest(int, const char *, struct tftphdr *, const char *);
@@ -117,7 +124,7 @@ sendfile(int fd, char *name, char *mode)
 	block = 0;
 	amount = 0;
 
-	signal(SIGALRM, timer);
+	bsd_signal(SIGALRM, timer);
 	do {
 		if (block == 0)
 			size = makerequest(WRQ, name, dp, mode) - 4;
@@ -132,7 +139,7 @@ sendfile(int fd, char *name, char *mode)
 			dp->th_block = htons((u_short)block);
 		}
 		timeout = 0;
-		(void) setjmp(timeoutbuf);
+		(void) sigsetjmp(timeoutbuf,1);
 send_data:
 		if (trace)
 			tpacket("sent", dp, size + 4);
@@ -221,7 +228,7 @@ recvfile(int fd, char *name, char *mode)
 	firsttrip = 1;
 	amount = 0;
 
-	signal(SIGALRM, timer);
+	bsd_signal(SIGALRM, timer);
 	do {
 		if (firsttrip) {
 			size = makerequest(RRQ, name, ap, mode);
@@ -233,7 +240,7 @@ recvfile(int fd, char *name, char *mode)
 			block++;
 		}
 		timeout = 0;
-		(void) setjmp(timeoutbuf);
+		(void) sigsetjmp(timeoutbuf,1);
 send_ack:
 		if (trace)
 			tpacket("sent", ap, size);
@@ -445,8 +452,8 @@ timer(int sig)
 	if (timeout >= maxtimeout) {
 		printf("Transfer timed out.\n");
 		errno = save_errno;
-		longjmp(toplevel, -1);
+		siglongjmp(toplevel, -1);
 	}
 	errno = save_errno;
-	longjmp(timeoutbuf, 1);
+	siglongjmp(timeoutbuf, 1);
 }
