@@ -74,6 +74,14 @@ static const char *rcsid = "tftp-hpa $Id: tftpd.c,v 1.4 1999/09/26 07:12:54 hpa
 
 #include "tftpsubs.h"
 
+void bsd_signal(int, void (*)(int));
+
+#ifndef HAVE_SIGSETJMP
+#define sigsetjmp(x,y)  setjmp(x)
+#define siglongjmp(x,y) longjmp(x,y)
+#define sigjmp_buf jmp_buf
+#endif
+
 #define	TIMEOUT 5		/* Default timeout (seconds) */
 #define TRIES   4		/* Number of attempts to send each packet */
 #define TIMEOUT_LIMIT (TRIES*(TRIES+1)/2)
@@ -85,9 +93,8 @@ static const char *rcsid = "tftp-hpa $Id: tftpd.c,v 1.4 1999/09/26 07:12:54 hpa
 #define EOPTNEG	8
 #endif
 
-extern	int errno;
 extern	char *__progname;
-struct	sockaddr_in s_in = { AF_INET };
+struct	sockaddr_in s_in;
 int	peer;
 int	timeout    = TIMEOUT;
 int	rexmtval   = TIMEOUT;
@@ -291,6 +298,8 @@ main(int argc, char **argv)
 		syslog(LOG_ERR, "socket: %m");
 		exit(1);
 	}
+	memset(&s_in, 0, sizeof s_in);
+	s_in.sin_family = AF_INET;
 	if (bind(peer, (struct sockaddr *)&s_in, sizeof (s_in)) < 0) {
 		syslog(LOG_ERR, "bind: %m");
 		exit(1);
@@ -622,7 +631,7 @@ validate_access(char *filename, int mode, struct formats *pf)
 }
 
 int	timeout;
-jmp_buf	timeoutbuf;
+sigjmp_buf	timeoutbuf;
 
 void
 timer(int sig)
@@ -631,7 +640,7 @@ timer(int sig)
 	timeout += rexmtval;
 	if (timeout >= maxtimeout)
 		exit(0);
-	longjmp(timeoutbuf, 1);
+	siglongjmp(timeoutbuf, 1);
 }
 
 /*
@@ -644,18 +653,18 @@ sendfile(struct formats *pf, struct tftphdr *oap, int oacklen)
 	struct tftphdr *ap;    /* ack packet */
 	int block = 1, size, n;
 
-	signal(SIGALRM, timer);
 	ap = (struct tftphdr *)ackbuf;
 
         if (oap) {
 	     timeout = 0;
-	     (void)setjmp(timeoutbuf);
+	     (void)sigsetjmp(timeoutbuf,1);
 oack:
 	     if (send(peer, oap, oacklen, 0) != oacklen) {
 		  syslog(LOG_ERR, "tftpd: oack: %m\n");
 		  goto abort;
 	     }
 	     for ( ; ; ) {
+	          bsd_signal(SIGALRM, timer);
 		  alarm(rexmtval);
 		  n = recv(peer, ackbuf, sizeof(ackbuf), 0);
 		  alarm(0);
@@ -691,7 +700,7 @@ oack:
 		dp->th_opcode = htons((u_short)DATA);
 		dp->th_block = htons((u_short)block);
 		timeout = 0;
-		(void) setjmp(timeoutbuf);
+		(void) sigsetjmp(timeoutbuf,1);
 
 send_data:
 		if (send(peer, dp, size + 4, 0) != size + 4) {
@@ -700,6 +709,7 @@ send_data:
 		}
 		read_ahead(file, pf->f_convert);
 		for ( ; ; ) {
+		        bsd_signal(SIGALRM, timer);
 			alarm(rexmtval);	/* read the ack */
 			n = recv(peer, ackbuf, sizeof (ackbuf), 0);
 			alarm(0);
@@ -748,7 +758,6 @@ recvfile(struct formats *pf, struct tftphdr *oap, int oacklen)
 	struct tftphdr *ap;    /* ack buffer */
 	int block = 0, acksize, n, size;
 
-	signal(SIGALRM, timer);
 	dp = w_init();
 	do {
 		timeout = 0;
@@ -762,7 +771,8 @@ recvfile(struct formats *pf, struct tftphdr *oap, int oacklen)
 		     ap->th_block = htons((u_short)block);
 		     acksize = 4;
 		}
-		(void) setjmp(timeoutbuf);
+		(void) sigsetjmp(timeoutbuf,1);
+		bsd_signal(SIGALRM, timer);
 send_ack:
 		if (send(peer, ackbuf, acksize, 0) != acksize) {
 			syslog(LOG_ERR, "tftpd: write(ack): %m");
@@ -770,6 +780,7 @@ send_ack:
 		}
 		write_behind(file, pf->f_convert);
 		for ( ; ; ) {
+		        bsd_signal(SIGALRM, timer);
 			alarm(rexmtval);
 			n = recv(peer, dp, PKTSIZE, 0);
 			alarm(0);
@@ -806,7 +817,7 @@ send_ack:
 	ap->th_block = htons((u_short)(block));
 	(void) send(peer, ackbuf, 4, 0);
 
-	signal(SIGALRM, justquit);      /* just quit on timeout */
+	bsd_signal(SIGALRM, justquit);      /* just quit on timeout */
 	alarm(rexmtval);
 	n = recv(peer, buf, sizeof (buf), 0); /* normally times out and quits */
 	alarm(0);
